/***********************************************
*Simulated annealing til løsning af DARP
*Forfatter: Grzegorz Nowak
*Filformat: UTF-8
************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>

#include "data/initdata.c"

#define bin char
//Antal kunder
#define n 200 
//antal ruter der initialiseres
#define m 100

#define N n*2
//En kunde må maksimalt køre i halvanden gange den direkte køreafstand
#define MRTfaktor 1.5

#define Tnul 20000
//#define Tnul 1000000
#define alpha 0.975
#define Tc 0.01
//#define SAmax 2000
//#define SAmax 1000
#define SAmax 3980
#define SAmin 200
#define nmin 200
#define exp 2.71828182846
#define ln2 0.693147181

#define TabuMax 20

//Hukommelsesbegrænsninger
#define antalRuter 200
#define punkterPerRute 100

//Slå forskellige indstillinger til [1] eller fra [0].
#define debug 0
#define testing 0
#define siman 0

#define PLUS(X) (X<0)?0:X
//max(X, 0)

int w[8]; //vægte

int afstand, frazone, tilzone, tmpzone;

struct stLoesninger
{
	int ruter[antalRuter][punkterPerRute]; //Rutedefinition
	int ruteS[antalRuter]; //ruteStørrelse = antal kunder i ruten
	int iAntalRuter; //antal ruter anvendt i løsningen
		
	//Beslutningsvariable
	int A[N]; //Ankomsttid i punkt [i] for den vogn der kører til punkt [i].
	int D[N]; //Afgangstid i punkt [i] for den vogn der kører fra punkt [i].
	int B[N]; //Service starttid i punkt [i]
	int W[N]; //Ventetiden for punkt [i]
	int Q[N]; //Last (anvendte sæder) i punkt [i] efter servicering
	int F[N]; //forward time slack bør være S[0].F[n]
	int P[N]; //Køretid for kunde der skal til punkt Pj
	int L[n]; //Køretid for kunder
	
	//objektfunktionsvariable
	
	int c1[antalRuter], c2[antalRuter], c3, c4[antalRuter], c5[antalRuter], c6[antalRuter], c7[antalRuter];
	//c1 = kørselsomkostninger, c2 = venteomkostninger, c3 = opstartsomkostninger, c4 = kunders køretid, c5 = brud på tidsvinduer, c6 = Brud på vognkapaciteten, c7 = MRT. 
	
	/*int c[antalRuter]; //Vognomkostninger for hver rute
	int q[antalRuter]; //Kapacitetsbrud for hver rute
	int w[antalRuter]; //tidsvindue brud for hver rute
	int t[antalRuter]; //brud på maksimal køretid for hver rute
	*/
	unsigned long omk; //samlede omkostninger
}S[3]; //S[0] er løsningen der arbejdes på, S[1] = den bedste løsning. Det kan udvides på den måde at den næstbedste løsning gemmes i S[2] osv.

//variabler
//int S[0].q[N]; //For hver kunde angives pladsforbrug i antal af sæder. Ved afhenting er værdien positiv, mens ved aflevering skal det være den nøjagtigt samme værdi med negativt fortegn.
int e[N], l[N]; //tidsvindue som hver kunde skal afhentes og afleveres indenfor. Afhentning: (e[i], l[i]) Aflevering: (e[n+i], l[n+i])
int d[N]; //Alle afhentnings- og afleveringspunkter er forbundet med en servicetid
int Qp[N]; //kapacitetsændring i et punkt for en kunde
int Qk[m]; //hver vogn har en kendt kapacitet
//int Rm[n]; //Hver kunde må maksimalt være i en vogn i en bestemt tidsramme (kan være den dobbelte direkte kørselstid)
//int Wm[N]; //Alle afhentnings- og afleveringspunkter er forbundet med en maksimal ventetid


int MRT[N]; //Maksimum ride time for hvert punkt

int z[N]; //liste over zoner til hvert punkt (både afhenting og aflevering).
int iWSum; //sum af ventetider temporær variabel. Anvendes som hjælpevariabel ved beregning af Fi

//Funktioner
void InitVar(void); //initialiser variabler fra datasæt
void InitLoesning(void);//initial løsning
void Programmering(int iRute); //Programmering af ankomst, ventetid, servicestarttid og afgang i iRute;

void fAfstand(int iRute, int i); //afstand til punktet iRute i ruten fra forgående punkt.
void OmbytRutePos(void); //udfører en positions ombytningsprocedure i samme rute (to punkter).
void TestOmbyt(void); //undersøger om ombytning er valid i forhold til afhentning og afleveringspunkt
void OmbytPos(void); //ombytter to positioner i samme rute (iOmbytPos og iRutePos)
int TestRute(int iRute); //tester om ruten er valid
void RuteTilTmp(int iRute); //kopierer indholdet af ruten til tmpruten
void NyRuteUdenPunkt(int iRute); //Lav ny iRute uden punktet der flyttes fra ruten

void FlytTilRute(void); //Flyt kunde fra en rute til en anden rute eller lav en helt ny rute til kunden
void OmbytPunktPar(void); //Vælg to tilfældige ruter og ombyt en kunde ét par punkter fra den ene rute med ét par punkter i den anden rute

long int fRute(int s, int r); //angiv omkostningerne for løsning s og rute r.
long int f(int s); //Beregner omkostningsfunktionen.
long int fReal(int s); //Beregner omkostninger uden vægte.

int FNul(int iRute); //Forward time slack (tidsforskydningsfaktor)
int Fi(int iRute, int i); //Forward time slack (tidsforskydningsfaktor)
int tij(int i, int j); //afstand mellem zone i og zone j (sørger for symmestrisk afstandstabel) m

int MAX(int X, int Y);
int MIN(int X, int Y);

void maksRuter(void);

//Main programvariabler
int  iPunkt, jPunkt, iRute, iRutePos, iTilRute, iRefRute, iOmbytPos, iRefPunkt, iAfhPos, iAfhInd, iAflPos, iAflInd, iTmpRute[100], iRuteAendret[2], iKundeAendret[2];

FILE *fpW;

int iAntalVogne, iStSl[1700][2], iTidligst, iSenest, iCount; //iStSl StartSlut på rute

int main (void)
{
	//Sæt vægte
	//c1 = kørselsomkostninger, c2 = venteomkostninger, c3 = opstartsomkostninger, c4 = kunders køretid, c5 = brud på tidsvinduer, c6 = Brud på vognkapaciteten, c7 = MRT. 
	w[1] = 8; w[2] = 1; w[3] = 110; w[4] = 3; w[5] = 0; w[6] = 1500;  w[7] = 1500;
	//w[1] = 1; w[2] = 1; w[3] = 30; w[4] = 1; w[5] = 1500; w[6] = 1500;  w[7] = 1500;
	
	//w[1] = 0; w[2] = 0; w[3] = 1000; w[4] = 0; w[5] = 1500; w[6] = 1500;  w[7] = 1500;
	//w[1] = 8; w[2] = 1; w[3] = 200; w[4] = 3; w[5] = 100; w[6] = 100;  w[7] = 100;
	//w[1] = 0; w[2] = 0; w[3] = 110; w[4] = 0; w[5] = 0; w[6] = 0;  w[7] = 0;
	//Fejlcheck
	if(m>n)
	{
		printf("\nDer er for mange ruter i forhold til kunder, programmet afsluttes!");
		return 1; //fejlkode 1
	}
	
	srand( time(NULL) );// initialize random seed
	unsigned long lTest00, lTest01, omk[SAmax], iOmkGns, iSigma;
	long iDelta;
	int iIterT, iIterAcc, iAntalOmk, iOpValg, iGlobalIter; //SA specifikke variable
	int iTabu[TabuMax][2], iTabuPos, bTabu; //iTabu har to indeks, [0] = kundeID [¡] = ændringsoperation
	
	double dT, dTny, x, dAlphaT; //SA specifikke variable
	
	
	time_t seconds;
  
	int i, j, k, p, test, tmp, tmp2, fejl;
	i=j=k=test=p=0;
		
	if(initAM()) return 1; //Hent afstandsmatricen
	if(initKunder()) return 1; //Hent kunder
	//if(initVogne()) return 1; //Hent vogne
	/*
	printf("\n init test: kunde 1 start tidsvindue: %i \n", stKB[1].iA);
	getchar();
	*/
	InitVar();//Initialiser variabler
		
	//SA algoritmen
	//Lav en initial løsning
	InitLoesning();//initial løsning

	char cResultatOutput[] = "../rawdata/output.txt"; //filnavn + sti
	if((fpW=fopen(cResultatOutput,"w"))==NULL) //åbner cFilnavn til læsning
	{
		printf("Kan ikke åbne %s , program afsluttes\n", cResultatOutput);
		return 1;
	}
	
	printf("\nInitialisering afsluttet\n");
	/*
	printf("\nAfstand fra 1 til 0 = %i\n", tij(z[S[0].ruter[219][1]], z[S[0].ruter[219][0]]));
	printf("\nTest MRT for 219 = %i", MRT[219]);
	getchar();
	*/
	/*
	S[0].ruter[0][0] = 1;
	S[0].ruter[0][1] = 0;
	S[0].ruter[0][2] = 3;
	S[0].ruter[0][3] = 2;
	*/
	//udskriv zonelisten
	/*printf("\n");
	for(i=0; i<4; i++)
		printf("zone %i = %i; ", i, z[i]);
	*/
	//printf("\nAfstand fra 1 til 0 = %i\n", tij(z[S[0].ruter[0][1]], z[S[0].ruter[0][0]]));
	
	//programmer alle ruter i løsningen
	
	for(iRute=0; iRute<S[0].iAntalRuter; iRute++)
	{
		//printf("\nRute %i programmeres", iRute);
		Programmering(iRute); //Programmering af iRute
	}
	
	printf("\nUdskriver rute 0:\n");
	for(i = 0; i<S[0].ruteS[0]; i++)
	{
		printf("%i-", S[0].ruter[0][i]);
	}
	
	lTest00 = f(0);
	lTest01 = fReal(0);
	
	printf("\nInitiale vægtede omkostninger = %i, ikke vægtede omkostninger = %i \n", lTest00, lTest01);
	fprintf(fpW, "\nInitiale vægtede omkostninger = %i, ikke vægtede omkostninger = %i \n", lTest00, lTest01);
	
	//Gem den initiale løsning som den bedste løsning
	S[0].omk = f(0); //gem de nuværende omkostnigner
	S[1] = S[0]; //S = S[1] får værdien fra den nuværende løsning S[0], hvorefter der kan laves om på S[0]
	S[2] = S[0]; //S[2] (Den bedste løsning) får værdien af den nuværende løsning S[0]
	
	//getchar();
	
	iIterT = 0; //Position for gentagelser for temperaturen T
	iIterAcc = 100; //antal accepterede løsninger ved den givne temperatur.
	iAntalOmk = 0; //antal omkostninger registreret til kølingsrate
	iTabuPos = 0; //Nulstil tabulistepositionen
	iGlobalIter = 0;
	
	dT = Tnul; //Start Temperatur for heuristikken, Denne temperatur vil falde indtil den når Tc
	//printf("Temperaturen er %f", dT);
	
	fejl=0; //NÅr fejl er lig nul, så bør der ikke være nogle fejl.
	
	//int iAntalVogne iStSl[1700][2]; //iStSl StartSlut på rute
	//indlæs startslut på alle ruter
	
	maksRuter();
	printf("\nDet maksimale antal ruter, der afvikles samtidigt = %i", iAntalVogne);
	//getchar();
	//Udskriv SA parametre i starten af output filen.
	fprintf(fpW, "Simulated annealing\n n = %i, m = %i, Tnul = %i, alpha = %f, Tc = %f, SAmax = %i", n, m, Tnul, alpha, Tc, SAmax);
	fprintf(fpW, "\n w1 = %i, w2 = %i, w3 = %i, w4 = %i, w5 = %i, w6 = %i, w7 = %i", w[1], w[2], w[3], w[4], w[5], w[6], w[7]);
	fprintf(fpW, "\nDet maksimale antal ruter, der afvikles samtidigt = %i", iAntalVogne);
	
	printf("\nSA starter på nu\n");
	seconds = time(NULL); //starttid på sim annealing
	iOpValg = 0;
	
	iGlobalIter = 0;
	iTabuPos = 0; //Nulstil tabulistepositionen
	
	while(iGlobalIter<1)
	{
		iIterT = 0; //Position for gentagelser for temperaturen T
		iIterAcc = 100; //antal accepterede løsninger ved den givne temperatur.
		iAntalOmk = 0; //antal omkostninger registreret til kølingsrate		
		dT = Tnul; //Start Temperatur for heuristikken, Denne temperatur vil falde indtil den når Tc
		
		while(dT > Tc) //SA heuristikken kører så længe at den nuværende temperatur er højere end slut temperaturen Tc
		//while(iIterAcc>=3) //SA heuristikken kører så længe at den nuværende temperatur er højere end slut temperaturen Tc
		{
			
			iIterAcc = 0; //antal accepterede løsninger nulstilles
			while(iIterT < SAmax) //lav SAmax gentagelser Ved konstant 
			{
				iIterT++;
				iRuteAendret[0]=iRuteAendret[1]=-1; //Ingen ruter ændret medmindre det angives
				iKundeAendret[0]=iKundeAendret[1]=-1;
				//switch(rand() % 3) //Der er tre mulige ryk Vælg et tilfældigt (uniformt) 
				switch(iOpValg)
				{
					case 0: OmbytRutePos();//Byt rundt på punkterne i en rute
									if(testing)printf("\n0 er valgt");
									break;
					case 1: FlytTilRute(); //Flyt kunde fra en rute til en anden rute eller lav en helt ny rute til kunden
									if(testing)printf("\n1 er valgt");
									break;
					case 2: OmbytPunktPar();//Ombyt et punktpar i en rute med et punktpar fra en anden rute
									if(testing)printf("\n2 er valgt");
									break;
				}
				//Programmer de ruter som er blevet ændret
				//hvilke ruter er blevet ændret?
				if(iRuteAendret[0]>=S[0].iAntalRuter || iRuteAendret[1]>=S[0].iAntalRuter)
				{
					printf("\n\nULOVLIG FEJL, en rute som ikke eksisterer er ændret ruterne: %i, %i antallet af ruter=%i\nTryk en tast for at fortsætte", iRuteAendret[0], iRuteAendret[1], S[0].iAntalRuter);
					fejl=1;
					getchar();
				}
				
				if(iRuteAendret[0]>=0)
				{
				//	printf("\nProgrammering af rute %i starter", iRuteAendret[0]);
					Programmering(iRuteAendret[0]);
					if(testing)
					{
						printf("\nOmkostninger for rute gammel: %i", iRuteAendret[0]);
						fRute(1, iRuteAendret[0]);
						printf("\nOmkostninger for rute ny: %i", iRuteAendret[0]);
						fRute(0, iRuteAendret[0]);
					}
				}
				
				if(iRuteAendret[1]>=0 && iRuteAendret[1] != iRuteAendret[0]) //programmer ruten kun, hvis den der forskellig fra den forgående programmerede rute: iRuteAendret[0].
				{
				//	printf("\nProgrammering af rute %i starter", iRuteAendret[1]);
					Programmering(iRuteAendret[1]);
					if(testing)
					{
						printf("\nOmkostninger for rute gammel: %i", iRuteAendret[1]);
						fRute(1, iRuteAendret[1]);
						printf("\nOmkostninger for rute ny: %i", iRuteAendret[1]);
						fRute(0, iRuteAendret[1]);
					}
				}
				
				//Beregn forskellen i opstartsomkostninger efter ruteprogrammeringen, så der ikke beregnes dobbeltstraf, hvis to ruter programmeres i samme iteration.
				S[0].omk -= w[3]*S[1].c3;
				S[0].omk += w[3]*S[0].c3;
				
				//S[0].omk = f(0);//beregn omkostningerne for den nuværende løsning; NY metode hvor omkostningerne opdateres ved ruteprogrammering
				
				if(testing) printf("\nNye omk = %i gamle omk = %i", S[0].omk, S[1].omk);
				
				iDelta = S[0].omk - S[1].omk; //Beregn ændringen
				
				if(testing) printf("Forskellen i omk = %i", iDelta);
				
				//Undersøg om rykket er tabu.
				bTabu = 0;
				//Udkommenter nedenstående for at slå tabulisten fra
				
				for(i=0; i<TabuMax; i++)//søg om kunden(erne) og rykket er tabu 
				{
					if(iTabu[i][0] == iKundeAendret[0] && iTabu[i][1] == iOpValg)
					{
						bTabu = 1;
					}
					if(iTabu[i][0] == iKundeAendret[1] && iTabu[i][1] == iOpValg)
					{
						bTabu = 1;
					}						
				}
				
				if(iDelta < 0) //hvis ændringen er mindre end nul, så er den nuværende løsningn bedre end den gamle
				{				
					if(S[0].omk < S[2].omk) //hvis den nuværende løsning er bedre end den bedste S[2].
					{
						iIterAcc++; //antallet af accepterede løsninger stiger
						printf("\nAntallet af ruter er nu: %i, temperaturen er %f og omk for S[2] = %i, S[1] = %i S[0] = %i\n", S[0].iAntalRuter, dT, S[2].omk, S[1].omk, S[0].omk);
						fprintf(fpW, "\nAntallet af ruter er nu: %i, temperaturen er %f og omk for S[2] = %i, S[1] = %i S[0] = %i\n", S[0].iAntalRuter, dT, S[2].omk, S[1].omk, S[0].omk);
						lTest00 = fReal(0);
						S[2] = S[0]; //Gem den nuværende løsning i den bedste lønsning
						S[1] = S[0]; //Den nuværende løsning gemmes i S !! Ingen test for tabuliste
					}
					else
					{
						if(bTabu<1) //rykket er ikke tabu gå videre med den nuværende løsning
						{
							iIterAcc++; //antallet af accepterede løsninger stiger
							S[1] = S[0]; //Den nuværende løsning gemmes.
							//Rykket bliver tabu
							if(iKundeAendret[0]>=0)
							{
								iTabuPos++;
								if(iTabuPos>=TabuMax-1)iTabuPos=0;
								iTabu[iTabuPos][0] = iKundeAendret[0];
								iTabu[iTabuPos][1] = iOpValg;
								
							}
							if(iKundeAendret[1]>=0)
							{
								iTabuPos++;
								if(iTabuPos>=TabuMax-1)iTabuPos=0;
								iTabu[iTabuPos][0] = iKundeAendret[0];
								iTabu[iTabuPos][1] = iOpValg;
							}
						}
						else //rykket er tabu, gå tilbage til forgående løsning
						{
							//printf("\nRykket er tabu!");
							//getchar();
							S[0] = S[1];
							iOpValg++; //løsning afvist vælg ny ændringsoperation
							if(iOpValg>2)//der er kun tre forskellige (0, 1, 2) ændringsoperationer
								iOpValg=0;
						}
					}
				}
				else
				{
					if(siman) //hvis forværende løsninger skal accepteres
					{
						x = (float)rand()/RAND_MAX; //Find en tilfældig værdi mellem 0 og 1 (uniformt)
						if(iDelta == 0)
						{
							iDelta = 1; //sørger for løsningen ikke næsten altid accepteres når forskellen i omkostningerne = 0;
						}
						if( x < pow(exp, (-iDelta/dT) ) && bTabu < 1 ) //Selvom løsningsværdien ikke er bedre end den forrige, vil den kunne accepteres med sandsynligheden  e^(-iDelta/dT)
						{
							iIterAcc++; //antallet af accepterede løsninger stiger
							S[1] = S[0]; //Den nuværende løsning gemmes i S
							
							//Rykket bliver tabu
						
							if(iKundeAendret[0]>=0)
							{
								iTabuPos++;
								if(iTabuPos>=TabuMax-1)iTabuPos=0;
								iTabu[iTabuPos][0] = iKundeAendret[0];
								iTabu[iTabuPos][1] = iOpValg;
							}
							if(iKundeAendret[1]>=0)
							{
								iTabuPos++;
								if(iTabuPos>=TabuMax-1)iTabuPos=0;
								iTabu[iTabuPos][0] = iKundeAendret[0];
								iTabu[iTabuPos][1] = iOpValg;
							}
							//printf("\nAntallet af ruter er nu: %i, temperaturen er %f og omk for S[2] = %i, S[1] = %i S[0] = %i\n", S[0].iAntalRuter, dT, S[2].omk, S[1].omk, S[0].omk);
							//lTest00 = fReal(0);
						}
						else
						{
							S[0] = S[1];//hvis løsningen ikke accepteres skal den nye løsning ikke anvendes. Indlæs forrige løsning.
							iOpValg++; //løsning afvist vælg ny ændringsoperation
							if(iOpValg>2)//der er kun tre forskellige (0, 1, 2) ændringsoperationer
								iOpValg=0;
						}
					}
					else //Hvis kun bedre løsninger accepteres
					{
						S[0] = S[1];//hvis løsningen ikke accepteres skal den nye løsning ikke anvendes. Indlæs forrige løsning.
						iOpValg++; //løsning afvist vælg ny ændringsoperation
						if(iOpValg>2)//der er kun tre forskellige (0, 1, 2) ændringsoperationer
							iOpValg=0;
					}
						
				}
				/*
				else //TEST
				{
					S[0] = S[1];//hvis løsningen ikke accepteres skal den nye løsning ikke anvendes. Indlæs forrige løsning.
				}
				*/
				if(testing)
				{
					printf("\nIteration = %i", iIterT);
					getchar();
				}
				
				//evaluering start
				
				//printf("\nAntallet af ruter er nu: %i, temperaturen er %f og omk for S[2] = %i, S[1] = %i S[0] = %i\n", S[0].iAntalRuter, dT, S[2].omk, S[1].omk, S[0].omk);
				/*printf("\nUdskriv alle ruter: \n");
				for( i=0; i<S[0].iAntalRuter; i++)
				{
					for(j=0; j<S[0].ruteS[i]; j++)
					{
						printf("%i ", S[0].ruter[i][j]);
					}
					printf("  ruteS[%i] = %i\n",i, S[0].ruteS[i]);
				}*/	
				//test alle ruter
				
				for(iRute=0; iRute<S[0].iAntalRuter; iRute++)
				{
					//printf("\nTester rute %i", iRute);
					if(TestRute(iRute))//tester om ruten er valid
						printf("\n\nUlovlig fejl i rute %i", iRute); 
				}
				if(fejl)
				{
					getchar();
					fejl=0;
				}
				//evaluering slut.
				/*printf("\nTest %i", iIterT);
				getchar();*/
				if(iIterAcc >= nmin && iIterT >= SAmin) //hvis accepterede løsninger er lig eller større end nmin(Stabil løsning) && antallet af iterationer er nået SAmin.
				{
					iIterT = SAmax; //når den stabile løsning er opnået skal afsluttes temperaturen ved at sætte iIterT til SAmax.
					//printf("\nDer afsluttes før SAmax");
				}
				omk[iAntalOmk] = S[0].omk;
				iAntalOmk++;				
			}
			
			//gennemsnitsomkostningen for temperaturen
			/*Skal fjernes hvis temp afhængig køling skal aktiveres
			iOmkGns = 0;
			for(i=0; i<iAntalOmk; i++)
			{
				iOmkGns += omk[i]; 
			}
			iOmkGns /=iAntalOmk;
			iSigma = 0;
			for(i=0; i<iAntalOmk; i++)
			{
				iSigma = (omk[i]-iOmkGns)*(omk[i]-iOmkGns);
			}
			iSigma = sqrt(iSigma/iAntalOmk);
			//printf("\niSigma = %i", iSigma);
	
			dAlphaT = 1 / (1 + ( (dT*ln2)/(3*iSigma) ));
			
			//printf("\ndAlphaT = %f", dAlphaT);
			
	
			dTny = dAlphaT*dT;
			
			if(dTny/dT < 0.8 || dTny/dT >=1)
			{
				dT = 0.8*dT;
			}
			else
			{
				dT = dTny;
			}*/
			
			dT = alpha*dT; //Temperatruen falder geometrisk
			printf("\nTemperatur = %f", dT);
			//getchar();
			
			iIterT = 0; //antal iterationer ved den nye dT er lig nul
			iAntalOmk = 0;//nulstilling af antal registrerede omkostninger
		}
		iGlobalIter++;
		printf("\nGlobalIteration = %i", iGlobalIter);
		fprintf(fpW, "\nGlobalIteration = %i", iGlobalIter);
		S[0] = S[1] = S[2]; //den nuværende løsning får værdien af den bedste løsning hidtil
	}
	//seconds = time(NULL);
	printf("\nSA slutter efter %ld sekunder", time(NULL) - seconds);
	fprintf(fpW, "\nSA slutter efter %ld sekunder", time(NULL) - seconds);
	
	S[0] = S[2]; //løsningen får værdien af den bedste løsning hidtil
	
	/*printf("\nAntallet af ruter er nu: %i, temperaturen er %f og omk for S[2] = %f, S[1] = %f S[0] = %f", S[0].iAntalRuter, dT, S[2].omk, S[1].omk, S[0].omk);
	
	*/
	fprintf(fpW, "\nUdskriv alle ruter: \n");
	for( i=0; i<S[0].iAntalRuter; i++)
	{
		for(j=0; j<S[0].ruteS[i]; j++)
		{
			fprintf(fpW, "%i ", S[0].ruter[i][j]);
		}
		fprintf(fpW,"  ruteS[%i] = %i\n",i, S[0].ruteS[i]);
	}
	
	/*printf("\nUdskriv alle ruter Bedste LØSNING: \n");
	for( i=0; i<S[0].iAntalRuter; i++)
	{
		for(j=0; j<S[0].ruteS[i]; j++)
		{
			printf("%i ", S[2].ruter[i][j]);
		}
		printf("  ruteS[%i] = %i\n",i, S[0].ruteS[i]);
	}		*/		
	//test alle ruter
	
	for(iRute=0; iRute<S[2].iAntalRuter; iRute++)
	{
		//printf("\nTester rute %i", iRute);
		if(TestRute(iRute))//tester om ruten er valid
		{
			printf("\n\nUlovlig fejl i rute %i, Tryk en tast for at komme videre", iRute); 
			getchar();
		}
	}
	
	lTest00 = f(0);
	lTest01 = fReal(0);
	
	printf("\nVægtede omkostninger = %i, ikke vægtede omkostninger = %i \n", lTest00, lTest01);
	fprintf(fpW, "\nVægtede omkostninger = %i, ikke vægtede omkostninger = %i \n", lTest00, lTest01);
	maksRuter();
	printf("\nDet maksimale antal ruter, der afvikles samtidigt = %i", iAntalVogne);
	fprintf(fpW, "\nDet maksimale antal ruter, der afvikles samtidigt = %i", iAntalVogne);
	iRute = 0;
	fclose(fpW);
	/*for(i=0; i<S[0].ruteS[iRute]; i++)
	{
		iPunkt = S[0].ruter[iRute][i];
	
		printf("Punkt = %i S[0].A[i] = %i S[0].B[i] = %i S[0].W[i] = %i S[0].D[i] = %i \n",iPunkt, S[0].A[iPunkt],  S[0].B[iPunkt], S[0].W[iPunkt], S[0].D[iPunkt]);
	}*/
	//SA algoritmen slut
	return 0;
}//MAIN END ****************************************************************************************************************************

void InitVar(void) // 0X = På servicetidX; 1X = Af servicetidX; 2X = kapacitetsforbrug; 3X = afhentningszone; 4X = Afleveringszone; 5X = MRT; 6-9X = de to tidsvinduer a,b,c,d
{
	int i;
	for(i=0; i<n; i++) //initialiser servicetider
	{
		d[i] = stKB[i].iPService; //servicetid er lig nul for alle punkter
		d[i+n] = stKB[i].iAService; //servicetid er lig nul for alle punkter
		/*printf("\n d[%i] = %i, d[%i + %i] = %i", i, d[i], i, n, d[i+n]);
		getchar();*/
	}
	
	for(i=0; i<n; i++) //initialiser tidsvinuder
	{
		e[i] = stKB[i].iA;
		l[i] = stKB[i].iB;
		e[i+n] = stKB[i].iC; 
		l[i+n] = stKB[i].iD; 
	}
	
	Qk[0] = 5; //Der anvendes kun én kapacitet for alle vogne
	
	//initialiser zoner til hvert punkt
	for(i=0; i<n; i++)//afheningspunkter
	{
		z[i] = stKB[i].iFraZone;
	}
	for(i=n; i<2*n; i++)
	{
		z[i] = stKB[i-n].iTilZone;
	}
	
	//Kapacitetsændring i et punkt
	
	for(i=0; i<n; i++)
	{
		Qp[i] = stKB[i].iAS; //servicetid er lig nul for alle punkter
		Qp[i+n] = -stKB[i].iAS; //servicetid er lig nul for alle punkter
		/*
		printf("\n Qp[%i] = %i, Qp[%i + %i] = %i", i, Qp[i], i, n, Qp[i+n]);
		getchar();
		*/
	}
	//initialiser MRT Maksimum ride time maksimal tid for kunden i en vogn S[0].A[2] - S[0].D[0], hvis der er to kunder
	for(i=0; i<n; i++)
	{
		MRT[i] = stKB[i].iMRT;
		/*beregning er udført i kb.txt
		MRT[i] = MRTfaktor*tij(z[i],z[i+n]);
		if(MRT[i]<MRTfaktor*tij(z[i],z[i+n]))//runder op hvis nødvendigt
			MRT[i]+=1;
		MRT[i+n] = MRT[i];*/
		/*
		printf("\nMRT[%i] = %i", i, MRT[i]);
		getchar();
		*/
	}
}

void InitLoesning(void)//initial løsning
{
	
	int i, j, k=0;
	
	int AntalKunderPerRute = n/m; //kunder fordeles på ruter
	int rest =  n-(AntalKunderPerRute*m); //rest af kunder, som skal fordeles på eksisterende ruter
	/*
	printf("\n rest = %i, AntalKunderPerRute = %i", rest, AntalKunderPerRute);
	getchar();
	*/
	for( i=0; i<m; i++)
	{
		if(rest>0)
		{
			S[0].ruteS[i] = AntalKunderPerRute + 1;
			//S[0].ruter[i][0]=stVS[0].B[i].iID; //tildel stardepot ID til starten af ruten. (depoter udelades)
			for(j=0; j<S[0].ruteS[i]; j++) 
			{
				S[0].ruter[i][j*2] = k;//fordeling af kunder ifølge en specifik regel.
				S[0].ruter[i][(j*2)+1] = k+n; //afleveringspunkt når n = 2 så har kunde 0 afhentning og aflevering i henholdsvist 0 og 2.
				k++;
				//printf("\n Der er en rest");
			}
			//S[0].ruter[i][((j-1)*2)+3]=stVS[0].B[i].iID+m; //tildel slutdepot ID til slutningen ruten. (depoter udelades)
			S[0].ruteS[i] = (AntalKunderPerRute + 1)*2;
			rest--;
		}
		else
		{
			S[0].ruteS[i] = AntalKunderPerRute;
			//printf("\nRutestørrelsen er %i", S[0].ruteS[i]);
			//S[0].ruter[i][0]=stVS[0].B[i].iID; //tildel stardepot ID til starten af ruten. (depoter udelades)
			for(j=0; j<S[0].ruteS[i]; j++) 
			{
				S[0].ruter[i][j*2] = k;//fordeling af kunder ifølge en specifik regel.
				S[0].ruter[i][(j*2)+1] = k+n; //afleveringspunkt når n = 2 så har kunde 0 afhentning og aflevering i henholdsvist 0 og 2.
				k++;
			}
			S[0].ruteS[i] = AntalKunderPerRute*2;
			//S[0].ruter[i][((j-1)*2)+3]=stVS[0].B[i].iID+m; //tildel slutdepot ID til slutningen ruten. (depoter udelades)
			//printf("\nRute %i initialiseres", i);
		}
	}
	S[0].iAntalRuter = m;
}

void Programmering(int iRute) //Programmering af ankomst, ventetid, servicestarttid og afgang i iRute;
{
	int i, j, p, tmp;
	//1) B0 får værdien e0. Servicetiden B for depotet på første rute "0" er lig starten af dens tidsvindue (Det tidligste tidspunktet hvor vognen er til rådighed). Der opereres ikke med tidsvinduer, da det antages at vognene altid er til rådighed, hvorfor B0 altid vil være 0
	//Note, alle servicetider i sættes til nul i første omgang
	
	//D0 får værdien e0. Afgangstiden D for vogn 0 vil svare til start tidsvindue i 0, idet det antages at afgang kan foregå lige efter ankomst
	
	S[0].A[S[0].ruter[iRute][0]] = e[S[0].ruter[iRute][0]];
	S[0].B[S[0].ruter[iRute][0]] = S[0].A[S[0].ruter[iRute][0]]; //starttid på service er lig ankomstiden
	S[0].D[S[0].ruter[iRute][0]] = S[0].A[S[0].ruter[iRute][0]] + d[S[0].ruter[iRute][0]]; //afgangstiden er lig ankomstiden plus servicetiden
	S[0].W[S[0].ruter[iRute][0]] = 0; //ingen ventetid i punkt 0
	
	/*
	printf("\n A0 = %i, W0 = %i, D0 = %i; A2 = %i, W2 = %i, D2 = %i; A1 = %i, W1 = %i, D1 = %i; A3 = %i, W3 = %i, D3 = %i;", S[0].A[0], S[0].W[0], S[0].D[0], S[0].A[2], S[0].W[2], S[0].D[2], S[0].A[1], S[0].W[1], S[0].D[1], S[0].A[3], S[0].W[3], S[0].D[3]);
	getchar();
	*/
	//1) slut
	//printf("\nPunkt %i: ", S[0].ruter[iRute][0]);
	i=0;
	//printf("S[0].A[i] = %i S[0].B[i] = %i S[0].W[i] = %i S[0].D[i] = %i \n", S[0].A[S[0].ruter[iRute][i]],  S[0].B[S[0].ruter[iRute][i]], S[0].W[S[0].ruter[iRute][i]], S[0].D[S[0].ruter[iRute][i]]);
	
	//2) beregn Ai Wi Bi og Di for alle punkter i den specificerede rute
	
	for(i=1; i<S[0].ruteS[iRute]; i++)
	{
		//printf("\nPunkt %i: ", S[0].ruter[iRute][i]);
		fAfstand(iRute, i); //sidste afstand i rute, punkt
		
		//printf("\nFra zone %i til zone %i er afstanden %i", frazone, tilzone, afstand);
	
		S[0].A[S[0].ruter[iRute][i]] = S[0].D[S[0].ruter[iRute][i-1]] + afstand; //Ankomst
		S[0].B[S[0].ruter[iRute][i]] = MAX(e[S[0].ruter[iRute][i]], S[0].A[S[0].ruter[iRute][i]]); //service start tid 
		S[0].W[S[0].ruter[iRute][i]] = S[0].B[S[0].ruter[iRute][i]]-S[0].A[S[0].ruter[iRute][i]]; //ventetid
		S[0].D[S[0].ruter[iRute][i]] = S[0].B[S[0].ruter[iRute][i]] + d[S[0].ruter[iRute][i]];//afgangstid
		
		//printf("S[0].A[i] = %i S[0].B[i] = %i S[0].W[i] = %i S[0].D[i] = %i \n", S[0].A[S[0].ruter[iRute][i]],  S[0].B[S[0].ruter[iRute][i]], S[0].W[S[0].ruter[iRute][i]], S[0].D[S[0].ruter[iRute][i]]);
	}
	/*
	printf("\n A0 = %i, W0 = %i, D0 = %i; A2 = %i, W2 = %i, D2 = %i; A1 = %i, W1 = %i, D1 = %i; A3 = %i, W3 = %i, D3 = %i;", S[0].A[0], S[0].W[0], S[0].D[0], S[0].A[2], S[0].W[2], S[0].D[2], S[0].A[1], S[0].W[1], S[0].D[1], S[0].A[3], S[0].W[3], S[0].D[3]);
	getchar();
	*/
	//2) slut
	
	//3) Beregn S[0].F[0]
	FNul(iRute); //Fnul beregenes for iRute
	//printf("\nS[0].F[0] for rute 0 er %i", S[0].F[0]);
	//3) slut
	
	//4) Beregn D0 = e0 + min (S[0].F[0], summen af ventetider fra første til sidste punkt i ruten)
	for(i = 0; i<S[0].ruteS[iRute]-1; i++) //S[0].ruteS[iRute]-1 da q (Det sidste punkt (afleveringspunkt) ikke er inkluderet i summen)
	{
		iWSum+=S[0].W[S[0].ruter[iRute][i]];
	}
	tmp = MIN( S[0].F[0] , iWSum);
	S[0].A[S[0].ruter[iRute][0]] = e[S[0].ruter[iRute][0]] + tmp;  //ankomsten forskydes i det første punkt på ruten
	S[0].B[S[0].ruter[iRute][0]] = MAX(e[S[0].ruter[iRute][0]], S[0].A[S[0].ruter[iRute][0]]); //service start tid 
	S[0].W[S[0].ruter[iRute][0]] = S[0].B[S[0].ruter[iRute][0]]-S[0].A[S[0].ruter[iRute][0]];
	S[0].D[S[0].ruter[iRute][0]] = S[0].A[S[0].ruter[iRute][0]] + d[S[0].ruter[iRute][0]] ;  //afgang er lig ankomst plus servicetid
	//printf("\ne0 = %i hvad er det %i", e[0], S[0].ruter[iRute][0] );
	//printf("\n Afgang fra rute %i er %i og summen af ventider er %i ",iRute, S[0].D[0], iWSum);
	//4) slut
	
	//5) Opdater Ai, Wi, Bi, og Di for hvert punkt i ruten
	//printf("\n\nVi er nået til punkt 5");
	for(i=1; i<S[0].ruteS[iRute]; i++)
	{
		//printf("\nPunkt %i: ", S[0].ruter[iRute][i]);
		fAfstand(iRute, i); //sidste afstand i (rute, punkt)
		
		//printf("\nFra zone %i til zone %i er afstanden %i", frazone, tilzone, afstand);
	
		S[0].A[S[0].ruter[iRute][i]] = S[0].D[S[0].ruter[iRute][i-1]] + afstand; //Ankomst
		S[0].B[S[0].ruter[iRute][i]] = MAX(e[S[0].ruter[iRute][i]], S[0].A[S[0].ruter[iRute][i]]); //service start tid 
		S[0].W[S[0].ruter[iRute][i]] = S[0].B[S[0].ruter[iRute][i]]-S[0].A[S[0].ruter[iRute][i]]; //ventetid
		S[0].D[S[0].ruter[iRute][i]] = S[0].B[S[0].ruter[iRute][i]] + d[S[0].ruter[iRute][0]];//afgangstid (mangler servicetiden)
		//printf("S[0].A[i] = %i S[0].B[i] = %i S[0].W[i] = %i S[0].D[i] = %i \n", S[0].A[S[0].ruter[iRute][i]],  S[0].B[S[0].ruter[iRute][i]], S[0].W[S[0].ruter[iRute][i]], S[0].D[S[0].ruter[iRute][i]]);
	}
	
	//5) slut
	
	//6) Køretider Li for hver kunde beregnes
	/*
	for(i = 0; i<S[0].ruteS[iRute]; i++)
	{
		if(S[0].ruter[iRute][i]<n)//Li beregnes kun for kunde ID som svarer til afhentningspunkt
		{
			S[0].L[S[0].ruter[iRute][i]] = S[0].B[S[0].ruter[iRute][i]+n] - S[0].D[S[0].ruter[iRute][i]];//Li = Bi+n - Di
			//printf("\nB = %i , D = %i", S[0].B[S[0].ruter[iRute][i+1]], S[0].D[S[0].ruter[iRute][i]]);
		}
	}
	*/
	//printf("\nKøretid for kunde 0 = %i for kunde 1 = %i", S[0].L[0], S[0].L[1]);
	//6) slut
	
	//7) For hvert punkt der svarer til afhentningspunkt for kunde j gør følgende:
	for(j=0; j<S[0].ruteS[iRute]; j++)
	{
		jPunkt = S[0].ruter[iRute][j]; // j er position i ruten, mens jPunkt er Punkt ID.
		if(jPunkt<n)
		{
			//a) beregn Fj
			Fi(iRute, j);
			//printf("\n\nPUNKT 7\nS[0].F[%i] = %i", j, S[0].F[j]);
			//a) slut
			
			//b) Sæt Bj = Bj + min (Fj, sum wp. sæt Dj = Bj + dj (dj er nul)
			iWSum=0;
			for(p=j+1; p<S[0].ruteS[iRute]; p++) //fjernet -1 i ( p<S[0].ruteS[iRute]-1 ) da det er til og med
			{
				iWSum+= S[0].W[S[0].ruter[iRute][p]];
			}
			S[0].B[jPunkt] += MIN( S[0].F[j] , iWSum);
			//S[0].B[jPunkt] += S[0].F[j];
			S[0].D[jPunkt] = S[0].B[jPunkt] + d[jPunkt]; //0 er erstattet med d[jPunkt]
			//printf("\nS[0].B[jPunkt] = %i, D = %i", S[0].B[jPunkt], S[0].D[jPunkt]);
			//b) slut
			
			//c) Opdaterer Ai Wi Bi og Di for alle punkter foran punkt j i ruten (Da det kun er disse der er påvirket af punkt b)
			//printf("\n\nPunkt c)");
			for (i = j+1; i<S[0].ruteS[iRute]; i++)
			{
				iPunkt = S[0].ruter[iRute][i];
				fAfstand(iRute, i); //sidste afstand i (rute, punkt)
				//printf("\nFra zone %i til zone %i er afstanden %i", frazone, tilzone, afstand);
				S[0].A[S[0].ruter[iRute][i]] = S[0].D[S[0].ruter[iRute][i-1]] + afstand; //Ankomst
				S[0].B[S[0].ruter[iRute][i]] = MAX(e[S[0].ruter[iRute][i]], S[0].A[S[0].ruter[iRute][i]]); //service start tid 
				S[0].W[S[0].ruter[iRute][i]] = S[0].B[S[0].ruter[iRute][i]]-S[0].A[S[0].ruter[iRute][i]]; //ventetid
				S[0].D[S[0].ruter[iRute][i]] = S[0].B[S[0].ruter[iRute][i]] + d[S[0].ruter[iRute][i]];//afgangstid
				
				//printf("\nS[0].A[i] = %i S[0].B[i] = %i S[0].W[i] = %i S[0].D[i] = %i \n", S[0].A[S[0].ruter[iRute][i]],  S[0].B[S[0].ruter[iRute][i]], S[0].W[S[0].ruter[iRute][i]], S[0].D[S[0].ruter[iRute][i]]);
			}
			//c) slut
			
			//printf("\n\nPunkt d)");
			//d) Opdater køretiden Li for alle kunder i som har afleveringspunkt efter punkt j
			/* flyttes udenfor løkken og køretiderne beregnes til slut
			for(i=j+1; i<S[0].ruteS[iRute];i++)
			{
				iPunkt=S[0].ruter[iRute][i];
				if(iPunkt>=n) //hvis afleveringspunkt
					S[0].L[iPunkt-n] =  S[0].B[iPunkt] - S[0].D[iPunkt-n]; //Li = Bi+n - Di
				
				//printf("\nB = %i , D = %i", S[0].B[iPunkt], S[0].D[iPunkt-n]);
			}
			*/
			//printf("\nS[0].L[%i] = %i",iPunkt-n, S[0].L[iPunkt-n]);
			//d) slut
		}
	}
	//Køretider
	for(i=0; i<S[0].ruteS[iRute];i++)
	{
		iPunkt=S[0].ruter[iRute][i];
		if(iPunkt>=n) //hvis afleveringspunkt
			S[0].L[iPunkt-n] =  S[0].B[iPunkt] - S[0].D[iPunkt-n]; //Li = Bi+n - Di
		
		//printf("\nB = %i , D = %i", S[0].B[iPunkt], S[0].D[iPunkt-n]);
	}
	//7) slut
	//8) Beregn brud på vognkapaciteten og begrænsningerne for tidsvinduer samt kørselstid  (rutelængden er der ikke begrænsning på).
	S[0].c1[iRute] = 0;
	S[0].c2[iRute] = 0; //ventetider
	//S[0].c3 = 0;
	S[0].c4[iRute] = 0;
	S[0].c5[iRute] = 0;
	S[0].c6[iRute] = 0; //nulstil kapacitetsbrudet for ruten, inden det beregnes
	S[0].c7[iRute] = 0;
	
	for(i = 0; i<S[0].ruteS[iRute]; i++)
	{
		iPunkt = S[0].ruter[iRute][i];
		S[0].c2[iRute] += S[0].W[iPunkt]; //ventetider
		
		if(iPunkt<n) //hvis afhentningspunkt (kundeID)
		{
			S[0].c4[iRute] += S[0].L[iPunkt]; //kundekøretid
			S[0].c7[iRute] += PLUS(S[0].L[iPunkt]- MRT[iPunkt]); //Brud på køretid MRT
		}
		
		S[0].c5[iRute] += PLUS(S[0].B[iPunkt]-l[iPunkt]); //tidsvinduer		
	}
	
	iPunkt = S[0].ruter[iRute][0]; //første punkt i ruten
	jPunkt = S[0].ruter[iRute][S[0].ruteS[iRute]-1];// sidste punkt i ruten
	S[0].c1[iRute] = S[0].D[jPunkt] - S[0].A[iPunkt] - S[0].c2[iRute]; //kørselsomkostninger Dvq -Av0 - ventetider
	
	//Brud på vognkapacitet
	iPunkt = S[0].ruter[iRute][0]; // iPunkt er kunde ID i rutepunktet
	S[0].Q[iPunkt] = Qp[iPunkt]; //Når ruten starter sættes kapacitetsforbruget til Qp for den første kunde i ruten
	if(S[0].Q[iPunkt]>Qk[0]) //hvis kapaciten i et punkt overstiges adderes et straf point til S[0].q[]
			S[0].c6[iRute]+= S[0].Q[iPunkt] - Qk[0];

	for(i=1; i<S[0].ruteS[iRute]; i++) //i er et punkt i ruten
	{
		iPunkt = S[0].ruter[iRute][i]; // iPunkt er kunde ID i rutepunktet
		
		S[0].Q[iPunkt]= S[0].Q[S[0].ruter[iRute][i-1]] + Qp[iPunkt]; //tilføj ændring i punktet Qp er positiv ved afhentning og negativ ved aflevering.
		if(S[0].Q[iPunkt]>Qk[0]) //hvis kapaciten i et punkt overstiges adderes et straf point til S[0].q[]
			S[0].c6[iRute]+= S[0].Q[iPunkt] - Qk[0];
	}
	
	//opstartsomkostningerne afhænger af antallet af ruter
	S[0].c3 = S[0].iAntalRuter;	
	
	//Opdatering af omkostninger
	//træk S[1].c1[iRute] * w1 +.... fra S[0] omk
	S[0].omk -= (w[1]*S[1].c1[iRute] + w[2]*S[1].c2[iRute] + w[4]*S[1].c4[iRute] + w[5]*S[1].c5[iRute] + w[6]*S[1].c6[iRute] + w[7]*S[1].c7[iRute]);
	//læg S[0].c1[iRute] * w1 til S[0].omk
	S[0].omk += w[1]*S[0].c1[iRute] + w[2]*S[0].c2[iRute] + w[4]*S[0].c4[iRute] + w[5]*S[0].c5[iRute] + w[6]*S[0].c6[iRute] + w[7]*S[0].c7[iRute];
	//Opdatering af omkostninger SLUT
	
	//printf("\nq = %i, 0 = %i 1=%i 2=%i 3=%i", S[0].q[iRute], S[0].Q[0], S[0].Q[1], S[0].Q[2], S[0].Q[3]);
	
	//c1 = kørselsomkostninger, c2 = venteomkostninger, c3 = opstartsomkostninger, c4 = kunders køretid, c5 = brud på tidsvinduer, c6 = Brud på vognkapaciteten, c7 = MRT.
	
	//S[0].c[iRute]+=S[0].D[ S[0].ruter[iRute][ S[0].ruteS[iRute]-1] ] - S[0].A[S[0].ruter[iRute][0]]; //Samlet køretid fra rute start A[0] til D[sidste punkt i ruten]
	
	//printf("\nS[0].w[0] = %i S[0].t[0] = %i, c[0] = %i", S[0].w[iRute], S[0].t[iRute], S[0].c[iRute]);
	//8) slut
	/*
	printf("\n A0 = %i, W0 = %i, D0 = %i; A2 = %i, W2 = %i, D2 = %i; A1 = %i, W1 = %i, D1 = %i; A3 = %i, W3 = %i, D3 = %i;", S[0].A[0], S[0].W[0], S[0].D[0], S[0].A[2], S[0].W[2], S[0].D[2], S[0].A[1], S[0].W[1], S[0].D[1], S[0].A[3], S[0].W[3], S[0].D[3]);
	printf("\nKøretid for ruten = %i", S[0].c1[iRute]);
	getchar();
	*/
}

void fAfstand(int iRute, int i)
{
	if(S[0].ruter[iRute][i-1]<n) //hvis det forgående punkt er et afhentningspunkt
		frazone = stKB[S[0].ruter[iRute][i-1]].iFraZone; //frazone for det forgående punkt
	else //ellers
		frazone = stKB[S[0].ruter[iRute][i-1]-n].iTilZone; //Tilzone fra det forgående afleveringspunkt.
	
	if(S[0].ruter[iRute][i]<n)//Hvis punktet er et afhentningspunkt
		tilzone = stKB[ S[0].ruter[iRute][i] ].iFraZone; //Frazone i det afhentningspunkt vi står i
	else	
		tilzone = stKB[ S[0].ruter[iRute][i]-n ].iTilZone; //tilzonen i det punkt vi står i, da det er et afleveringspunkt

	if(frazone > tilzone) //sørg for at det altid er fra lille værdi til stor værdi, da matricen antages at være symmetrisk
	{
		tmpzone = frazone;
		frazone = tilzone;
		tilzone = tmpzone;
	}
	afstand = iAM[tilzone][frazone];
}

void OmbytRutePos(void) //udfører en positions ombytningsprocedure i samme rute (to punkter).
{
	//printf("\nOmbytRutePos starter\n");
	int i, j;
	//Lav rækkefølge om i ruten
	//1) Vælg en tilfældig rute i løsningen
	
	 //for(i=0; i<40; i++)
  iRute = rand() % (S[0].iAntalRuter) ; //vælg en rute som ligger mellem 0 og antallet af ruter i løsningen (læg mærke til at at hvis der er en rute i løsningen vil outputtet altid være rute 0 når S[0].iAntalRuter = 1;
 	//if(debug)printf(" %i ", iRute);
  //1)slut
  
  //1.1hvis ruten kun har en kunde afslut
  if(S[0].ruteS[iRute] <=2)
  {
	}
	else
	{
		//2)Vælg et tilfældigt punkt i ruten
		iRutePos = rand() % S[0].ruteS[iRute]; //vælg en tilfældig position i ruten
		//iRutePos = 3;
		//2)slut
	
		//punkt 3 bør kun køres hvis der er flere end én kunde i ruten
		//3)Vælgt et tilfældigt ombytningspunkt i ruten (hvis afhentingspunkt, før afleveringspunkt; hvis afleveringspunkt efter afhentningspunkt
		if(S[0].ruter[iRute][iRutePos]<n)//afhentningspunkt i position iRutePos
		{
			if(debug)printf("\nAFHENTNINGSPUNKT");
			i=1;
			while(S[0].ruter[iRute][i]!=S[0].ruter[iRute][iRutePos]+n)
				i++; //find afleveringsposition
			iOmbytPos= rand() % i; //iOmbytPos skal være før afleveringspunktet
			if(debug)printf("\niOmbytPos = %i, afleveringsposition=%i", iOmbytPos, i);
		}
		else //afleveringspunkt i position iRutePos
		{
			if(debug)printf("\nAFLEVERINGSPUNKT");
			i=iRutePos;
			
			while(S[0].ruter[iRute][i]!=S[0].ruter[iRute][iRutePos]-n)
				i--; //find afhentningsposition
			i++;
			iOmbytPos= (rand()%(S[0].ruteS[iRute]-i)) + i; //iOmbytPos skal være efter afhentningspositionen
			if(iOmbytPos>=S[0].ruteS[iRute] || iOmbytPos<i)
				printf("\n\n MEGA FEJL iOmbyt har fået en ulovlig værdi");
			if(debug)printf("\niOmbytPos = %i, afhetningsposition=%i", iOmbytPos, i);
		}
		TestOmbyt(); //Se om ombytning kan lade sig gøre i forhold til afhentning og afleveringspunkter, hvis det ikke kan lade sig gøre, sættes iOmbyt = iRutePos
		//3) slut
		
		//4) Ombyt iOmbytPos og iRutePos hvis de er forskellige
		OmbytPos(); //tager højde for om iOmbyt er forskellig fra iRutePos
		//4) slut
		if(iOmbytPos!=iRutePos) //hvis der sker en ombytning
		{
			
			iRuteAendret[0]=iRute; //Fortæller hvilke ruter der er blevet ændret (I dette tilfælde kun iRute)
			//info til tabu
			if(S[0].ruter[iRute][iRutePos]<n)
				iKundeAendret[0] = S[0].ruter[iRute][iRutePos];
			else
				iKundeAendret[0] = S[0].ruter[iRute][iRutePos]-n;
			
			if(testing)
			{
				printf("\nPunkterne i rute %i er byttet rundt", iRute);
				//udskriv den nye rute
				printf("\nNy rute:\n");
				for(j = 0; j<S[0].ruteS[iRute]; j++)
				{
					printf("%i-", S[0].ruter[iRute][j]);
				}
				printf("\nGammel rute:\n");
				//udskriv hvordan den så ud før
				for(j = 0; j<S[1].ruteS[iRute]; j++)
				{
					printf("%i-", S[1].ruter[iRute][j]);
				}			
			}
		}
	}
}

void TestOmbyt(void)
{
	int i=iOmbytPos;
	if(iOmbytPos<iRutePos) //test om ombytning kan lade sig gøre (ombytningspunkt før iRutePos)
	{
		if(S[0].ruter[iRute][i]<n)
		{
			if(debug)printf("\nOmbytning med et afhentningspunkt noooo");
			while(S[0].ruter[iRute][i]!=S[0].ruter[iRute][iOmbytPos]+n)
				i++; //find afhentningsposition
		}
		else
		{
			if(debug)printf("\nOmbytning med et afleveringspunkt til venstre for iRutePos (KAN ALTID LADE SIG GØRE)");
			i=iRutePos;
		}
		if(i<iRutePos)
		{
			iOmbytPos=iRutePos;
			if(debug)printf("\nFEJL ombytning kan ikke lade sig gøre");
		}
		if(debug)printf("\niOmbytPos = %i, i=%i", iOmbytPos, i);
	}
	else if(iOmbytPos>iRutePos)
	{
		if(debug)printf("\nOmbytningspunkt er efter iRutePos");
		i=iOmbytPos;
		
		if(S[0].ruter[iRute][i]>=n) //Afleveringspunkt
		{
			if(debug)printf("\nOmbytning med et afleveringspunkt noooo");
			
			while(S[0].ruter[iRute][i]!=S[0].ruter[iRute][iOmbytPos]-n) //find Afleveringspunktets afhentningspunkt
				i--; //find afhentningsposition
			if(debug)printf("\nAfleveringspunktets afhentningspunkt er %i", i);
		}
		else
		{
			if(debug)printf("\nOmbytning med et afhentningspunkt til højre for iRutePos (KAN ALTID LADE SIG GØRE)");
			i=iRutePos;
		}
		if(i>iRutePos)
		{
			iOmbytPos=iRutePos;
			if(debug)printf("\nFEJL ombytning kan ikke lade sig gøre");
		}
		if(debug)printf("\niOmbytPos = %i, i=%i", iOmbytPos, i);
	}				
	else 
	{
		if(debug)printf("\nOmbytningspunkt og referencepunkt er lig hinanden");
	}		
}

void OmbytPos(void)
{
	int tmp;
	if(iOmbytPos != iRutePos)
  {
  	tmp = S[0].ruter[iRute][iOmbytPos]; //gem værdien i iOmbytPos
  	S[0].ruter[iRute][iOmbytPos] = S[0].ruter[iRute][iRutePos]; //flyt punktet fra iRutePos til iOmbytPos
  	S[0].ruter[iRute][iRutePos] = tmp; //gem værdien som var i iOmbytPos i iRutePos
  }
}

int TestRute(int iRute) //tester om ruten er valid
{
	int i, j, k;
	
	j=0;
	for(i=0; i<(S[0].ruteS[iRute]/2); i++)
	{
		
		while(S[0].ruter[iRute][j]>=n) //find første opsamlingspunkt
		{
			if(j>=n*2)
			{		
				printf("\n\nFejl i rute %i der er ikke nok afhentingspunkter", iRute);
				return 1; //returnerer fejlkode 1
				break;
			}
			j++;
		}
		//printf("\nOpsamlingspunkt er i %i", j);
		k=j;
		while(S[0].ruter[iRute][j]+n!=S[0].ruter[iRute][k]) //find afleveringspunkt
		{
			if(k>=(n*2))
			{
				printf("\n\nFejl i Rute %i afleveringspunkt mangler", iRute);
				return 2; //returnerer fejlkode 2
				break;
			}
			k++;
		}
		//printf("\nAfleveringspunkt er i %i", k);
		//printf("\ni=%i", i);
		j++;
	}
	return 0;
}

void RuteTilTmp(int iRute) //kopierer indholdet af ruten til tmpruten
{
	int i;
	for(i=0; i<S[0].ruteS[iRute]; i++)
	{
		iTmpRute[i] = S[0].ruter[iRute][i];
	}
}

void NyRuteUdenPunkt(int iRute) //Lav ny iRute uden punktet der flyttes fra ruten
{
	int i, j;
	j=0;
	for(i=0; i<S[0].ruteS[iRute]; i++) 
	{
		if(i!=iAfhPos && i!=iAflPos)
		{
			S[0].ruter[iRute][j] = iTmpRute[i];
			j++;
		}
	}
}

void FlytTilRute(void) //Flyt kunde fra en rute til en anden rute eller lav en helt ny rute til kunden
{	
	//printf("\nFlytTilRute starter\n");
	int i, j;
	
	/*flyttet
	iRuteAendret[0] = -1;
	iRuteAendret[1] = -1;
	*/
	
	//1) Udvælg en tilfældig rute
	iRute = rand() % (S[0].iAntalRuter) ; //vælg en rute som ligger mellem 0 og antallet af ruter i løsningen (læg mærke til at at hvis der er en rute i løsningen vil outputtet altid være rute 0 når S[0].iAntalRuter = 1;
	
	//1.2) Vælg rute som punkterne skal flyttes til, som er forskelligt fra iRute
	iTilRute = iRute;
	while(iRute == iTilRute)
	{
		iTilRute = rand() % (S[0].iAntalRuter+1); //+1 da der gerne må oprettes en ny rute
	}
	
	//1.3) Hvis ruten kun har to punkter og skal flyttes over i en ny rute, så gør intet
	if(iTilRute == S[0].iAntalRuter && S[0].ruteS[iRute] == 2)
	{
		//printf("\nRuteflytning gavner ikke noget, afslut FlytTilRute!\n");
	}
	else
	{
		//2)Vælg et tilfældigt punkt i ruten
		iRutePos = rand() % S[0].ruteS[iRute]; //vælg en tilfældig position i ruten
		
		//3)Find punktets tilsvarende afhentings eller afleveringspunkt
		if(S[0].ruter[iRute][iRutePos]<n) //afhentningspunkt
		{
			//printf("\nAfhentningspunkt");
			iAfhPos = iRutePos;
			iRefPunkt = S[0].ruter[iRute][iRutePos]+n;
			//printf("\niAfhPos = %i, iRefPunkt = %i", iAfhPos, iRefPunkt);
			iRutePos++;
			//printf("\nS[0].ruter[iRute][iRutePos] = %i", S[0].ruter[iRute][iRutePos]);
			while(S[0].ruter[iRute][iRutePos]!=iRefPunkt) //find positionen på afleveringspunktet
			{
				iRutePos++;
				if(iRutePos>=S[0].ruteS[iRute])
				{
					printf("\n\nUlovlig FEJL i FlytTilRute! - Ved søgning efter afl. punkt");
					getchar();
					break;
				}
			}
			iAflPos = iRutePos;
			//printf("\niAflPos = %i", iAflPos);
		}
		else //afleveringspunkt
		{
			//printf("\nAfleveringspunkt");
			iAflPos = iRutePos;
			iRefPunkt = S[0].ruter[iRute][iRutePos]-n;
			iRutePos--;
			//printf("\nIref = %i",iRefPunkt);
			while(S[0].ruter[iRute][iRutePos]!=iRefPunkt) //find positionen på afhentningspunktet
			{
				
				if(iRutePos<0)
				{
					printf("\n\nUlovlig FEJL i Flyt TilRute!!  - Ved søgning efter afh. punkt for afleveringspunktet %i i ruteposition %i", S[0].ruter[iRute][iAflPos], iAflPos);
					printf("\nUdskriver rute:\n");
					for(i = 0; i<S[0].ruteS[iRute]; i++)
					{
						printf("%i-", S[0].ruter[iRute][i]);
					}
					getchar();
					break;
				}
				iRutePos--;
			}
			iAfhPos = iRutePos;
		}
		
		//printf("\nAfh = %i Afl = %i", iAfhPos, iAflPos);
		
		//4)Vælg rute som punkterne skal flyttes til
		// Er flyttet til start 
		
		//5) En helt ny rute laves
		if(iTilRute>=S[0].iAntalRuter) 
		{
			//printf("\nEn helt ny rute laves");
			S[0].ruter[iTilRute][0] = S[0].ruter[iRute][iAfhPos]; //afhentningspunktet sættes
			S[0].ruter[iTilRute][1] = S[0].ruter[iRute][iAflPos]; //afleveringspunktet sættes
			S[0].ruteS[iTilRute] = 2; //Rutestørrelsen sættes
			
			//Kopier iRute til tmp rute
			RuteTilTmp(iRute);
			NyRuteUdenPunkt(iRute); //Lav ny iRute uden punktet der flyttes fra ruten
			S[0].ruteS[iRute] -= 2; //Rutestørrelsen bliver mindre i iRute
			
			S[0].iAntalRuter++;
			
			//Nedenstående ruter ændres
			iRuteAendret[0] = iRute;
			iRuteAendret[1] = iTilRute;
			
			//info til tabu
			iKundeAendret[0] = S[0].ruter[iTilRute][0]; //special case
			
			if(testing)
			{
				printf("\nNy rute er lavet: %i", iTilRute);
				printf("\nPunkter taget fra: %i", iRute);
			}
		}
		else if(iTilRute == iRute)
		{
			//printf("\nSamme rute valgt som iRute, gør intet");
		}	
		else //Punktet flyttes over i en eksisterende rute
		{
			//printf("\nPunktet flyttes over i en eksisterende rute");
			//Kopier iTilRutetil tmp rute 
			RuteTilTmp(iTilRute);
			
			//Sæt ny rutestørelse
			S[0].ruteS[iTilRute] += 2; //Rutestørrelsen sættes
			
			//Find indsættelsespunkt til afhentning
			iAfhInd = rand() % (S[0].ruteS[iTilRute]-1); //-1 fordi et afhentningspunkt ikke kan være i slutningen af ruten
			//Find indsættelsespunkt til aflevering
			iAflInd = (rand() % (S[0].ruteS[iTilRute]-(iAfhInd+1)) ) + (iAfhInd+1); //find position på afleveringspunktet, som skal ligge et tilfældigt sted mellem afhentningspunkt og slutnignen af ruten.
			if(iAfhInd == iAflInd) //kan fjernes når det hele er robust ROB
				printf("\n\nUlovlig FEJL i \"FlyTilRute\" iAfhInd er lig iAflInd (burde ikke kunne ske)");
			j=0;
			for(i=0; i<S[0].ruteS[iTilRute]; i++)
			{
				if(i==iAfhInd) //sæt afhentingspunktet ind i den nye rute
				{
					S[0].ruter[iTilRute][i] = S[0].ruter[iRute][iAfhPos];
					iKundeAendret[0] = S[0].ruter[iRute][iAfhPos]; //info til tabu
				}
				else if(i==iAflInd) //sæt afleveringspunktet ind i den nye rute
					S[0].ruter[iTilRute][i] = S[0].ruter[iRute][iAflPos];
				else //når det hverken er et afleveringspunkt eller afhentningspunkt, skal de resterende punkter i den originale rute indsættes
				{
					S[0].ruter[iTilRute][i] = iTmpRute[j];
					j++;
				}
			}		
			//ryd op i iRute
			RuteTilTmp(iRute); //iRute lægges over i tmprute
			NyRuteUdenPunkt(iRute); //iRute 
			S[0].ruteS[iRute] -= 2; //Rutestørrelsen bliver mindre i iRute	
			
			//Nedenstående ruter ændres
			//if(S[0].ruteS[iRute]>0)
			iRuteAendret[0] = iRute;
			iRuteAendret[1] = iTilRute;			 
		}
		
		//Er størrelsen på iRute nul, skal ruten fjernes void FjernRute(void);
		if(S[0].ruteS[iRute]<=0)
		{
			//printf("\nRute %i skal fjernes", iRute);
			if(iRute+1!=S[0].iAntalRuter) //hvis iRute ikke er den sidste rute i rutelisten, skal hullet i rutelisten lappes.
			{
				//vælg sidste rute i rutelisten og sæt den lig iRute
				iRefRute = S[0].iAntalRuter-1;  // minus 1 fordi antallet af ruter ikke svarer til rutelistens index, som starter ved nul.
				if(iRefRute!=iRute) //Sker kun hvis de to ruter ikke er ens
				{
					S[0].ruteS[iRute] = S[0].ruteS[iRefRute]; //sæt rutestørrelsen.
					for(i=0; i<S[0].ruteS[iRute]; i++) //Kopier den sidste rute ind på iRute's plads.
					{
						S[0].ruter[iRute][i] = S[0].ruter[iRefRute][i];
					}
				}
				else
				{
					iRuteAendret[0] = -1; //
					iKundeAendret[0] = -1; //annuller tabuinfo
				}
			}
			//til sidst dekrementeres S[0].iAntalRuter
			S[0].iAntalRuter--;
			if(iTilRute >= S[0].iAntalRuter) //hvis tilruten flyttes tilbage, skal iTilrute ikke programmeres, da den ikke findes
				iRuteAendret[1] = -1;
			if(iRute >= S[0].iAntalRuter)
				iRuteAendret[0] = -1;
		}
	}
}

void OmbytPunktPar(void)//Vælg to tilfældige ruter og ombyt to tilfældige kunder fra begge ruter.
{
	//printf("\nOmbytPunktPar starter\n");
	int i, tmp;
	if(S[0].iAntalRuter>1) //Det kan antages at der altid er mere end 1 rute, hvorved denne "hvis" sætning kan spares væk.
	{
		i=0;
		
		//printf("\n S[0].iAntalRuter = %i", S[0].iAntalRuter);
		
		iRute = rand() % S[0].iAntalRuter;
		//printf("\nRute 1 valgt = %i", iRute);
		
		iTilRute = iRute; //Sørg for at nedenstående løkke starter;
		while(iRute == iTilRute)//sørg for at de to ruter ikke er ens
		{
			iTilRute = rand() % S[0].iAntalRuter;
			if(iTilRute>=S[0].iAntalRuter)
			{
				printf("\n\n MEGA FEJL!! rute 2 har fået en for høj værdi KAN IKKE LADE SIG GØRE i OmbytPunktPar!!!");
				iTilRute = iRute;
			}
		}
		if(S[0].ruteS[iRute] == 2 && S[0].ruteS[iTilRute] == 2) // hvis begge ruter kun indeholder 1 kunde, skal de ikke ombyttes.
		{
			iRuteAendret[0] = -1;
			iRuteAendret[1] = -1;
			//printf("\nOmbytning af kunder i OmbytPunktpar er ikke nødvendig!\ntryk en tast for at fortsætte!");
			//getchar();			
		}
		else
		{
			//printf("\nRute 2 valgt = %i", iTilRute);
			//printf("\nAntallet af ruter er %i", S[0].iAntalRuter);
			
			iPunkt = rand() % (S[0].ruteS[iRute]-1); //afhentingspunkt i iRute.
			
			//printf("\niPunkt = %i og rutesS[%i] = %i", iPunkt, iRute, S[0].ruteS[iRute]);
			if(S[0].ruter[iRute][iPunkt]<n) //afhentningspunkt
			{	
				//printf("\nipunkt er et afhetningspunkt");	
				iAfhPos = iPunkt;
				iAflPos = iPunkt+1;
				while(S[0].ruter[iRute][iAfhPos]+n != S[0].ruter[iRute][iAflPos]) //find aflpos.
					iAflPos++;
			}
			else //afleveringspunkt.
			{
				//printf("\nipunkt er et afleveringspunkt");	
				iAflPos = iPunkt;
				iAfhPos = iAflPos;
				while(S[0].ruter[iRute][iAflPos]-n != S[0].ruter[iRute][iAfhPos]) //find afhpos
					iAfhPos--;			
			}
				
			//gem punkterne fra iRute i den midlertidige rute
			iTmpRute[0] = S[0].ruter[iRute][iAfhPos]; 
			iTmpRute[1] = S[0].ruter[iRute][iAflPos];
			iKundeAendret[0] = S[0].ruter[iRute][iAfhPos]; //tabu info
			//printf("\nPunkterne %i og %i fra rute %i i positionerne %i %i gemmes i iTmpRute", iTmpRute[0], iTmpRute[1], iRute, iAfhPos, iAflPos);
			
			jPunkt = rand() % S[0].ruteS[iTilRute]; //afhentingspunkt i iTilRute
			
			if(S[0].ruter[iTilRute][jPunkt]<n) //afhentningspunkt
			{	
				//printf("\nipunkt er et afhetningspunkt");	
				iAfhInd = jPunkt;
				iAflInd = jPunkt+1;
				while(S[0].ruter[iTilRute][iAfhInd]+n != S[0].ruter[iTilRute][iAflInd]) //find aflpos
					iAflInd++;
			}
			else //afleveringspunkt
			{
				//printf("\nipunkt er et afleveringspunkt");	
				iAflInd = jPunkt;
				iAfhInd = iAflInd;
				while(S[0].ruter[iTilRute][iAflInd]-n != S[0].ruter[iTilRute][iAfhInd]) //find afhpos
					iAfhInd--;			
			}
			
			//Gem punkterne fra iTilRute i iRute
			S[0].ruter[iRute][iAfhPos] = S[0].ruter[iTilRute][iAfhInd];
			S[0].ruter[iRute][iAflPos] = S[0].ruter[iTilRute][iAflInd];
			iKundeAendret[1] = S[0].ruter[iTilRute][iAfhInd];//tabu info.
			//Gem punkterne fra iRute (ligger i iTmpRute listen) i iTilRute
			S[0].ruter[iTilRute][iAfhInd] = iTmpRute[0];
			S[0].ruter[iTilRute][iAflInd] = iTmpRute[1];
			//ombytning udført
			//ruter ændret:
			iRuteAendret[0] = iRute;
			iRuteAendret[1] = iTilRute;
		}
	}
}
int FNul(int iRute) //Forward time slack (tidsforskydningsfaktor)
{
	int tmp, tmp2, iWSum, j, p, iPunkt;
	int i = 0;
	S[0].F[i] = 1000;
	
	for(j = i; j<n*2; j++)
	{
		iWSum=0;
		for(p=i+1; p<=j; p++)
		{
			iPunkt=S[0].ruter[iRute][p];
			iWSum+=S[0].W[iPunkt];
		}
		iPunkt=S[0].ruter[iRute][j];
		
		//Beregning af Køretiden "P" for kunde som skal til iPunkt
		/*if(iPunkt<n)
			S[0].P[iPunkt] = 0; //afhentningspunkt køretid = 0
		else
			S[0].P[iPunkt] = S[0].B[iPunkt] - S[0].D[iPunkt-n]; //afleveringspunkt => køretid = service starttid minus afgang fra afhentningspunkt
		*/
		tmp = PLUS(l[iPunkt]-S[0].B[iPunkt]);
		//tmp = MIN( l[iPunkt]-S[0].B[iPunkt] , MRT[iPunkt] - S[0].P[iPunkt]);
		tmp2 = iWSum + tmp;
		if(tmp2<S[0].F[i])
			S[0].F[i] = tmp2;
		//printf("\nj=%i lj=%i, Bj=%i, MRTj=%i, Pj=%i S[0].F[0]=%i iPunkt = %i", j, l[iPunkt], S[0].B[iPunkt], MRT[iPunkt], S[0].P[iPunkt], S[0].F[i], iPunkt);
	}
}

long int fRute(int s, int r) //angiv omkostningerne for løsning s og rute r.
{
	/*
	unsigned long c[10];
	c[1] += S[s].c1[r];
	c[2] += S[s].c2[r];
	c[4] += S[s].c4[r];
	c[5] += S[s].c5[r];
	c[6] += S[s].c6[r];
	c[7] += S[s].c7[r];
	*/
	printf("\nc[1] = %i c[2] = %i c3 = %i c[4] = %i c[5] = %i c[6] = %i c[7] = %i", S[s].c1[r], S[s].c2[r], S[s].c3, S[s].c4[r], S[s].c5[r], S[s].c6[r], S[s].c7[r]);

}

long int f(int s) //Beregner omkostningsfunktionen. //bruges i princippet kun for løsning nul
{ 
//c1 = kørselsomkostninger, c2 = venteomkostninger, c3 = opstartsomkostninger, c4 = kunders køretid, c5 = brud på tidsvinduer, c6 = Brud på vognkapaciteten, c7 = MRT. 
	int i;
	unsigned long c[10];
	unsigned long iTotal=0, tmp;
		
	c[1] = c[2] = c[3] = c[4] = c[5] = c[6] = c[7] = 0;
	
	for(i=0; i<S[s].iAntalRuter; i++)
	{
		tmp = iTotal;
		/*
		c[1] += S[s].c1[1];
		c[2] += S[s].c1[2];
		c[4] += S[s].c1[4];
		c[5] += S[s].c1[5];
		c[6] += S[s].c1[6];
		c[7] += S[s].c1[7];
		*/
		iTotal += w[1]*S[s].c1[i] + w[2]*S[s].c2[i] + w[4]*S[s].c4[i] + w[5]*S[s].c5[i] + w[6]*S[s].c6[i] + w[7]*S[s].c7[i];
		if(iTotal<tmp)
		{
			printf("\nUlovlig fejl unsigned long ikke lang nok");
			getchar();
		}
	}
	
	//iTotal = w[1]*c[1] + w[2]*c[2] + w[4]*c[4] + w[5]*.c[5] + w[6]*c[6] + w[7]*c[7];
	
	S[s].c3 = S[s].iAntalRuter;
	c[3] = S[s].c3;
	iTotal += w[3]*c[3];
	return iTotal;
}

long int fReal(int s) //Beregner omkostninger uden vægte
{
	int i, j;
	
	unsigned long c[10];
	unsigned long iTotal = 0;
	
	c[1] = c[2] = c[3] = c[4] = c[5] = c[6] = c[7] = 0;
	
	S[s].c3 = S[s].iAntalRuter; //opstartsomkostninger
	iTotal += S[s].c3;

	for(i=0; i<S[s].iAntalRuter; i++)
	{
		c[1] += S[s].c1[i];
		c[2] += S[s].c2[i];
		c[4] += S[s].c4[i];
		c[5] += S[s].c5[i];
		c[6] += S[s].c6[i];
		c[7] += S[s].c7[i];
		/*if(S[s].c5[i]>0 || S[s].c6[i]>0 || S[s].c7[i]>0)
		{
			printf("\nHård begrænsning er brudt c5 = %i c6 = %i c7 = %i i løsning %i i rute %i ", S[s].c5[i], S[s].c6[i], S[s].c7[i], s, i);
			printf("\n udskriv rute %i\n:", i);
			for(j=0; j<S[0].ruteS[i]; j++)
			{
				printf("%i-", S[0].ruter[i][j]);
			}
			printf("\n");
			for(j=0; j<S[0].ruteS[i]; j++)
			{
				iPunkt = S[0].ruter[i][j];
				printf("Punkt = %i S[0].A[i] = %i S[0].B[i] = %i S[0].W[i] = %i S[0].D[i] = %i \n",iPunkt, S[0].A[iPunkt],  S[0].B[iPunkt], S[0].W[iPunkt], S[0].D[iPunkt]);
			}
			getchar();
		}
		*/
		/*
		iTotal += S[s].c1[i] + S[s].c2[i] + S[s].c4[i] + S[s].c5[i] + S[s].c6[i] + S[s].c7[i];
		printf("\n S[s].c1[i] = %i + S[s].c2[i] = %i + S[s].c3[i] = %i + S[s].c4[i] = %i + S[s].c5[i] = %i + S[s].c6[i] = %i + S[s].c7[i] = %i", S[s].c1[i], S[s].c2[i],S[s].c3, S[s].c4[i], S[s].c5[i],  S[s].c6[i], S[s].c7[i]);*/
	}
	iTotal += c[1] + c[2] + c[4] + c[5] + c[6] + c[7];
	
	printf("\n c1 = %i c2 = %i c3 = %i c4 = %i c5 = %i c6 = %i c7 = %i", c[1], c[2], S[s].c3, c[4], c[5],  c[6], c[7]);
	fprintf(fpW, "\n c1 = %i c2 = %i c3 = %i c4 = %i c5 = %i c6 = %i c7 = %i", c[1], c[2], S[s].c3, c[4], c[5],  c[6], c[7]);
	
	return iTotal;	
}

int Fi(int iRute, int i) //Forward time slack (tidsforskydningsfaktor)
{
	int tmp, tmp2, iWSum, j, p, iPunkt;
	
	S[0].F[i] = 1000;
	
	for(j = i; j<S[0].ruteS[iRute]; j++)
	{
		iWSum=0;
		for(p=i+1; p<=j; p++)
		{
			iPunkt=S[0].ruter[iRute][p];
			iWSum+=S[0].W[iPunkt];
			//printf("\nBeregning af iWSum: iPunkt = %i, S[0].W[iPUnkt] = %i", iPunkt, S[0].W[iPunkt]);
		}
		iPunkt=S[0].ruter[iRute][j];
		
		//Beregning af Køretiden "P" for kunde som skal til iPunkt
		if(iPunkt<n)
			S[0].P[iPunkt] = 0; //afhentningspunkt køretid = 0
		else
			S[0].P[iPunkt] = S[0].B[iPunkt] - S[0].D[iPunkt-n]; //afleveringspunkt => køretid = service starttid minus afgang fra afhentningspunkt
		
		tmp = PLUS(MIN( l[iPunkt]-S[0].B[iPunkt] , MRT[iPunkt] - S[0].P[iPunkt]));
		//tmp = MIN( l[iPunkt]-S[0].B[iPunkt] , MRT[iPunkt] - S[0].P[iPunkt]);
		tmp2 = iWSum + tmp;
		if(tmp2<S[0].F[i])
			S[0].F[i] = tmp2;
	}
	/*printf("\nj=%i lj=%i, Bj=%i, MRTj=%i, Pj=%i S[0].F[%i]=%i iPunkt = %i, iWSum = %i", j, l[iPunkt], S[0].B[iPunkt], MRT[iPunkt], S[0].P[iPunkt],i, S[0].F[i], iPunkt, iWSum);
	getchar();
	*/
}

int tij(int i, int j) //afstand mellem zone i og zone j (sørger for symmestrisk afstandstabel)
{
	if(i<j)
		return iAM[j][i];
	else if(i>j)
		return iAM[i][j];
	else
		return 0;
}

int MAX(int X, int Y)
{
	int result;
	result=(X>Y)?X:Y;
	return result;
}

int MIN(int X, int Y)
{
	int result;
	result=(X<Y)?X:Y;
	return result;
}

void maksRuter(void)
{
	int i, j;
	iTidligst = 2400;
	iSenest = 0;
	for(i=0; i<S[0].iAntalRuter; i++)
	{
		iPunkt = S[0].ruter[i][0]; //startpunkt på ruten
		jPunkt = S[0].ruter[i][S[0].ruteS[i]-1]; //slutpunkt på ruten
		iStSl[i][0] = S[0].A[iPunkt];
		iStSl[i][1] = S[0].D[jPunkt];
		
		if(iTidligst>iStSl[i][0])
			iTidligst = iStSl[i][0];
		if(iSenest<iStSl[i][1])
			iSenest = iStSl[i][1];		
		/*printf("\nStartpunkt på ruten er: %i slut: %i, starttid = %i, sluttid = %i", iPunkt, jPunkt, iStSl[i][0], iStSl[i][1]);
		getchar();*/
	}
	
	printf("\niTidligst = %i, iSenest = %i", iTidligst, iSenest);
	fprintf(fpW, "\niTidligst = %i, iSenest = %i", iTidligst, iSenest);
	
	iAntalVogne = 0;
	iCount = 0;
	for(i=iTidligst; i<=iSenest; i++) //test alle tider hvor der er vogne på vejen
	{
		for(j=0; j<S[0].iAntalRuter; j++)
		{
			if(i>= iStSl[j][0] && i<= iStSl[j][1]) //tæl hvor mange ruter der er igang på tidspunkt i
			{
				iCount++;
			}				
		}
		
		if(iAntalVogne < iCount)
			iAntalVogne = iCount;
		
		iCount = 0;		
	}
}

